      PROGRAM disp_sma1_2r 
c program to obtain the optimum undeground velocity structure for
c the given dispersion curve of Rayleigh wave based on the down
c hill simplex method combined with the simulated anealing approach.
c
c                            by T. Yokoi, IISEE, BRI(July 4, 2005)
c
c Modified for the progress report file output (July 8, 2010)
c Modified for outputting DHS for error estimation (November 29,2010)
c 
c Revised with fixed layer's thickness for shallow exploration
c                                                  (May, 5,2017)
      parameter(lmax=25,np0=50,mp0=51,nf0=512,j00=20,n_hist=50)
      real z(lmax),roh(lmax),vp(lmax),vs(lmax),vp1(lmax),vs1(lmax),
     *     roh1(lmax),ap(lmax),ae(lmax),qp(lmax),qs(lmax),y0(4),
     *     yij(42),h(lmax),h1(lmax),fr(nf0),vr_obs(nf0),sd_obs(nf0),
     *     vr_cal(nf0),vmx(lmax),vmn(lmax),hmx(lmax),tk,tol,
     *     hmn(lmax),pmx(np0),pmn(np0),p(mp0,np0),x(np0),y(mp0)
cccccc
      real pst(np0+1,n_hist)
cccccc
      integer n_judge(j00),n_roh,n_vp,IL,nf,itr,ia
      character CDUM*80,filen*72,fnam1*25,fnam2*25,fnam3*25,fnam4*25
      character cdrv*10,cwkf*80
      EXTERNAL  RAYMRX,funk
      COMMON    /Temp/tk /Range/pmn,pmx
      COMMON    /Layer/h,roh,vp,vs,IL
      COMMON    /Flag/n_roh,n_vp
      COMMON    /Obs/nf,fr,vr_obs,sd_obs
      COMMON    /Crtl/ap,ae,y0,yij,tol,itr,ia
      COMMON    /Seed/idum
      DATA  pi2/6.283185/ 
      data  cdrv/'./cca_wkf/'/ 
      data  TOL,ITR,IA/0.01,100,0/ t0,a,c,ntemp,j0/1.,0.5,1.0,10000,5/
      write(6,*)'+---------------------------------------------------+'
      write(6,*)'+                                                   +'
      write(6,*)'+                    Disp_sma1                      +'
      write(6,*)'+                                                   +'
      write(6,*)'+ Program to obtain the optimum undeground velocity +'
      write(6,*)'+ structure for the given dispersion relation of    +'
      write(6,*)'+ Rayleigh wave.                                    +'
      write(6,*)'+                                                   +'
      write(6,*)'+ The used method is a combination of the down hill +'
      write(6,*)'+ simplex method (Nelder & Mead (1965)) and the     +'
      write(6,*)'+ very fast simulated annealing method (Ingber      +'
      write(6,*)'+ (1989)).                                          +'
      write(6,*)'+                                                   +'
      write(6,*)'+ The subroutine DSPRAY and DSPMRX published in     +'
      write(6,*)'+ "Seismological Algorithm" are used directly.      +'
      write(6,*)'+ AMOEBA and AMOTRY published in "Numerical Recipe" +'
      write(6,*)'+ are also used, but with significant modification  +'
      write(6,*)'+ for the adaptation with the very fast simulated   +'
      write(6,*)'+ annealing method.                                 +'
      write(6,*)'+                                                   +'
      write(6,*)'+ By the combination with the down hill simplex     +'
      write(6,*)'+ method, the very fast simulated annealing method  +'
      write(6,*)'+ is gotten much faster.                            +'
      write(6,*)'+                                                   +'
c     write(6,*)'+ I do not put any requirement for usage to anyone, +'
c     write(6,*)'+ except that they must do it under their own       +'
c     write(6,*)'+ responsibility. I neither guarantee the safe      +'
c     write(6,*)'+ running of the program, nor take any kind of      +'
c     write(6,*)'+ responsibility for any loss or damage due to usage+'
c     write(6,*)'+ that users may suffer.                            +'
c     write(6,*)'+                                                   +'
c     write(6,*)'+ Information on bugs and the ways to fix them are  +'
c     write(6,*)'+ welcome all the time.                             +'
c     write(6,*)'+                                                   +'
      write(6,*)'+                                       July 6, 2005+'
      write(6,*)'+    CopyRight by Toshiaki Yokoi, IISEE, BRI, Japan.+'
      write(6,*)'+---------------------------------------------------+'
c Prameter file reading
c      data  eps0/0.06/
c flags for roh and vp
c  n_roh    : 1=by Ludwig et al(1970), 
c             vp=1.11*vs+1.29
c             0=fixed to the initial values
c  n_vp     : 1=by Kitzunezaki et al(1990), 
c             roh=1.2475+0.399*vp-0.026*vp**2
c             0=fixed to the initial values
c      data n_roh,n_vp/1,1/   
c flags for output to CRT
c  ini_flg  : Initial Velocity Structure Model             1=yes
c  ndsp_flg : Observed Dispersion Relation                 1=yes
c  n_err    : Error at each iteration                      1=yes
c  kflg     : Missfit at each temp. change                 1=yes
c  jflg     : Missfit at each itration with the same temp. 1=yes
c  n_vs     : Vs value        (n_vs=layer number, 0=no output)
c  n_th     : Thickness value (n_th=layer number, 0=no output)
c      data  ini_flg,ndsp_flg/1,0/  kflg,jflg/0,1/
c     *      n_vs,n_th/0,0/
c
      cwkf=cdrv//'data/results/'
      filen=cwkf(1:23)//'progress.dat'
      write(6,*)filen
c
      open(19,file=filen,status='unknown')
      cwkf=cdrv//'prm/'
      filen=cwkf(1:14)//'disp_sma1_2r.prm'
      write(6,*)filen
      open(11,file=filen,status='old')
      read(11,*)idum,t0,a,c,ntemp,j0
      read(11,*)eps0
      read(11,*)n_roh,n_vp
      read(11,*)ini_flg,ndsp_flg,n_err
      read(11,*)kflg,jflg
      read(11,*)n_vs,n_th
      read(11,*)fnam1
      read(11,*)fnam2
      read(11,*)fnam3
      read(11,*)fnam4
      close(11)
C Observed dispersion relation reading
      cwkf=cdrv//'data/results/'
      filen=cwkf(1:23)//fnam2
      write(6,*)filen
      open(2,file=filen,status='old')
      read(2,'(a80)')cdum
      do i=1,nf0
       read(2,'(2f10.3)',end=10) fr(i),vr_obs(i) 
       vr_obs(i)=vr_obs(i)*1.e-3
       sd_obs(i)=1.e-3
      enddo
   10 nf=i-1
      close(2)
      if(ndsp_flg.eq.1) then
       write(6,'(6x,2a10)')' Frequency','  Velocity' 
       write(6,'(2F12.3)') (fr(i),vr_obs(i),i=1,nf)
      endif
      nr_min=nf
      vr_min=vr_obs(nf)
      nr_max=1
      vr_max=vr_obs(1)
      do i=1,nf
       if(vr_min.gt.vr_obs(i)) then
        vr_min=vr_obs(i)
        nr_min=i
       endif
       if(vr_max.lt.vr_obs(i)) then
        vr_max=vr_obs(i)
        nr_max=i
       endif
      enddo
      th_min=vr_obs(nf)/fr(nf)/3.
C INITIAL STRUCTURE MODEL READING
      filen=cdrv//'prm/'//fnam1
      write(6,*)filen
      open(1,file=filen,status='old')
      READ(1,'(A30)')  MODEL 
      READ(1,*)  IL
      dep_av=0.
      dep_tp=0.
      dep_bm=0.
      do L=1,IL                             
c      READ(1,*,END=15)ROH(L),VP(L),h(L),vmn(L),vmx(L)    ! layer thickness h_fixed
c
       if(L.eq.1) then
        h(L)=th_min
       else
        h(L)=max(0.001,vr_max/fr(nr_max)/2./real(IL))        ! Empirical relation
       endif
       if(L.ge. 2) dep_tp=dep_tp+h(L-1)
       if(L.lt.IL) then
        dep_bm=dep_bm+h(L)
       else if(L.eq.IL) then
        dep_bm=dep_bm+h(L-1)
       endif
c      write(6,'(i2,2f8.5)')L,dep_tp,dep_bm
       nn_d=0
       sm_n=0.
       do i=1,nf
        rmd_3=vr_obs(i)/fr(i)/3.
        if(dep_tp.lt.rmd_3.and.rmd_3.le.dep_bm) then
         sm_n=sm_n+vr_obs(i)
         nn_d=nn_d+1
        endif
       enddo
       if(nn_d.eq.0) then
        if(L.eq.1) then
         vs(L)=vr_min
        else
         vs(L)=vs(L-1)*1.2
        endif
       else       
        vs(L)=sm_n/real(nn_d)
       endif
       vmn(L)=vs(L)/1.2
       vmx(L)=vs(L)*1.2
c
       if(n_vp.eq.1) then
        vp(L)=1.11*vs(L)+1.29
       endif
       if(n_roh.eq.1) then
        roh(L)=1.2475+0.399*vp(L)-0.026*vp(L)**2
       endif 
      enddo
   15 close(1)
      h(IL)=999.0                                   ! Bedrock
      if(ini_flg.eq.1) then
       write(6,'(a32,a30)')'Initial values randomly produced',
     *                     '; Range of random fluctuation '
       write(6,'(4a8,a16)')'Thicknes',' Density',
     *  '      Vp','      Vs','      Vs        '
       write(6,'(6f8.5)')(h(L),roh(L),vp(L),vs(L),
     *                              vmn(L),vmx(L),L=1,IL-1)
       write(6,'(8x,5f8.5)')(roh(L),vp(L),vs(L),
     *                              vmn(L),vmx(L),L=IL,IL)
      endif
cjump for theoretical calculation
c      goto 999
c preparation for simplex method
      do L=1,IL                     ! search only on Vs       
       pmn(L)=vmn(L)             
       pmx(L)=vmx(L)
      enddo
      NP=IL                         ! number of unknowns = number of layers 
      MP=NP+1
      do m=1,mp
       do L=1,IL
        p(m,L)=vs(L)
       enddo
      enddo
      do m=2,mp
       p(m,m-1)=p(m,m-1)*1.1
      enddo
c     write(6,'(6f6.3)')((p(m,n),n=1,np),m=1,mp)
      do m=1,MP
       do n=1,NP
        x(n)=p(m,n)
       enddo
       y(m)=funk(x)
c      write(6,*)m,y(m)
      enddo
c Temp. change
      do k=1,ntemp
       tk=temprt(t0,k-1,a,c)
c       WRITE(6,*)k,'  Tk=',TK
c iteration for same temp.
c
       call amoeba(p,y,MP,MP0,NP,NP0,IL,j0,funk,iter)
c 
       m1=1
       do m=2,mp
        if(y(m).lt.y(m1)) then
	    m1=m
	   endif
       enddo
       err=abs(y(m1))
       if(n_err.eq.1) then 
        write(6,'(i5,f16.10)')k*j0,err
       endif
c output to the progress report file
       do L=1,IL
        vs(L)=p(m1,L)
        if(n_vp.eq.1) then
         vp(L)=1.11*vs(L)+1.29
        endif
        if(n_roh.eq.1) then
         roh(L)=1.2475+0.399*vp(L)-0.026*vp(L)**2
        endif 
       enddo
       h(IL)=999.
       write(19,*)'***'
	  write(19,'(i5,f16.10)')k*j0,err
       write(19,'(5x,4a15)')'  Thicknes(Km) ','Density(g/cm^3)',
     *                     '     Vp(Km/sec)','     Vs(Km/sec)'
       write(19,'(I5,4F15.6)')(L,h(L),roh(L),vp(L),vs(L),L=1,IL)
c
       if(kflg.eq.1) then
        write(6,'(i5,a4,f8.3,a5,f16.10)')k,' Tk=',tk,' err=',err
       endif
       if(err.lt.eps0) then
        goto 20
       endif
c      write(6,*)err_1,err_2,iden
cccccc Mar.8,2013 from here
       if(k.le.n_hist) then
        do j=1,np
         pst(j,k)=p(m1,j)
        enddo
        pst(np+1,k)=er
       else
        do jk=1,n_hist-1 
         do j=1,np+1
          pst(j,jk)=pst(j,jk+1)
         enddo
        enddo
        do j=1,np
         pst(j,n_hist)=p(m1,j)
        enddo
        pst(np+1,n_hist)=er
        judge=0
        do jk=1,n_hist-1
         do j=1,np+1
          if(abs(pst(j,jk)-pst(j,jk+1)).gt.eps0) then
           judge=1
          endif
         enddo
        enddo
        do j=1,np+1
         if(abs(pst(j,1)-pst(j,n_hist)).gt.eps0) then 
          judge=1
         endif
        enddo
        if(judge.eq.0) goto 20
       endif
cccccc Mar.8, 2013 to here
      enddo
c
   20 continue   
c
      do L=1,IL
       vs(L)=p(m1,L)
       if(n_vp.eq.1) then
        vp(L)=1.11*vs(L)+1.29
       endif
       if(n_roh.eq.1) then
        roh(L)=1.2475+0.399*vp(L)-0.026*vp(L)**2
       endif 
      enddo
      h(IL)=999.
C Result output to Display & file
  999 continue
      filen=cwkf(1:23)//fnam3
      write(6,*)filen
      write(6,'(5x,4a15)')'  Thicknes(Km) ','Density(g/cm^3)',
     *                    '     Vp(Km/sec)','     Vs(Km/sec)'
      write(6,'(I5,4F15.6)')(L,h(L),roh(L),vp(L),vs(L),L=1,IL)
      open(1,file=filen,status='unknown')
      write(1,'(5x,4a15)')'  Thicknes(Km) ','Density(g/cm^3)',
     *                    '     Vp(Km/sec)','     Vs(Km/sec)'
      write(1,'(I5,4F15.6)')(L,h(L),roh(L),vp(L),vs(L),L=1,IL)
      close(1)
c
      write(19,*)'***        ***'
      write(19,*)'*** Result ***'
      write(19,'(5x,4a15)')'  Thicknes(Km) ','Density(g/cm^3)',
     *                     '     Vp(Km/sec)','     Vs(Km/sec)'
      write(19,'(I5,4F15.6)')(L,h(L),roh(L),vp(L),vs(L),L=1,IL)
      close(19)
C Theoretical Dispersion Curve for the final model
      cmn=vs(1)*0.5
      cmx=vs(IL)*1.2
      dc=(cmx-cmn)/1000.
      do i=1,nf
       W=PI2*fr(i)
       CALL  RAYDSP(RAYMRX,H,ROH,VP,VS,AP,AE,IL,W,CMN,CMX,DC,
     *                        TOL,ITR,IA,C,UD,EKD,Y0,YIJ,IER1)
       vr_cal(i)=C
      enddo
c
      filen=cwkf(1:23)//fnam4
      write(6,*)filen
      open(2,file=filen,status='unknown')
      write(2,'(2x,3a20)')'#   Frequency(Hz)   ',
     *       '   Observed Velocity',' Calculated Velocity'
      do i=1,nf
       write(2,'(4(4x,F12.3,4x))')
     *         fr(I),vr_obs(I),vr_cal(I),vr_cal(I)/vr_obs(I)
      enddo
c      write(2,'(a3,e13.6)')'er=',err
      close(2)
c
      STOP
      END
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      function funk(x)
      parameter(lmax=25,lmax2=50,nf0=512)
      real x(lmax2),h(lmax),roh(lmax),vp(lmax),vs(lmax),fr(nf0),
     *     vr_obs(nf0),sd_obs(nf0),vr_cal(nf0),y0(4),yij(42),
     *     ap(lmax),ae(lmax),tk,tol
      integer n_roh,n_vp,IL,nf,itr,ia
      COMMON /Layer/h,roh,vp,vs,IL
      COMMON /Flag/n_roh,n_vp
      COMMON /Obs/nf,fr,vr_obs,sd_obs
      COMMON /Crtl/ap,ae,y0,yij,tol,itr,ia
      EXTERNAL  RAYMRX
      DATA  pi2/6.283185/
c      write(6,*)' funk'
      do L=1,IL
       vs(L)=x(   L)
       if(n_vp.eq.1) then
        vp(L)=1.11*vs(L)+1.29
       endif
       if(n_roh.eq.1) then
        roh(L)=1.2475+0.399*vp(L)-0.026*vp(L)**2
       endif 
      enddo
      h(IL)=999.0
C Theoretical Dispersion Curve for Initial Model
      cmn=vs(1)*0.5
      cmx=vs(IL)*1.2
      dc=(cmx-cmn)/1000.
c      write(6,*)(vp(L),L=1,IL)
      do i=1,nf
       W=PI2*fr(i)
       CALL  RAYDSP(RAYMRX,H,ROH,VP,VS,AP,AE,IL,W,CMN,CMX,DC,
     *                        TOL,ITR,IA,C,UD,EKD,Y0,YIJ,IER1)
       vr_cal(i)=C
      enddo
c Misfit Function
      sum=0.
      do i=1,nf
c       sum=sum+abs((vr_obs(i)-vr_cal(i))/vr_obs(i))
c       sum=sum+abs((vr_obs(i)-vr_cal(i))/sd_obs(i))
       sum=sum+abs(vr_obs(i)-vr_cal(i))**2
      enddo  
      funk=sqrt(sum)/real(nf)
c      write(6,'(a5,f16.12)') ' err=',funk
      return
      end
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      function temprt(t0,k,a,c)
      integer k
      temprt=t0*exp(-c*k**a)
      return
      end      
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE amoeba(p,y,mp,mp0,np,np0,ndim,ntol,funk,iter)
c Numerical Recipe', modified for simulated anealing approach by T. Yokoi 
      INTEGER iter,mp,ndim,np,NMAX,ITMAX
      REAL ftol,p(mp0,np0),y(mp0),funk,TINY,tk,tol
      PARAMETER(NMAX=50,ITMAX=10000,TINY=1.e-10,lmax=25,lmax2=50,nf0=512
     *          ,big=1.e+30)
      EXTERNAL RAYMRX,funk
CU    USES amotry,funk
      INTEGER i,ihi,ilo,inhi,j,m,n,n_roh,n_vp,IL,nf,itr,ia
      REAL rtol,sum,swap,ysave,ytry,psum(np0),amotry
	real pmn(lmax2),pmx(lmax2),h(lmax),roh(lmax),vp(lmax),vs(lmax),
     *     fr(nf0),sd_obs(nf0),vr_obs(nf0),y0(4),yij(42),
     *     ap(lmax),ae(lmax)
      COMMON /Temp/tk /Range/pmn,pmx 
      COMMON /Layer/h,roh,vp,vs,IL
      COMMON /Flag/n_roh,n_vp
      COMMON /Obs/nf,fr,vr_obs,sd_obs
      COMMON /Crtl/ap,ae,y0,yij,tol,itr,ia
      COMMON /Seed/idum
      itol=0
      iter=0
1     do 12 n=1,ndim
        sum=0.
        do 11 m=1,ndim+1
          sum=sum+p(m,n)
11      continue
        psum(n)=sum
12    continue
2     ilo=1
      if (y(1).gt.y(2)) then
        ihi=1
        inhi=2
      else
        ihi=2
        inhi=1
      endif
      do 13 i=1,ndim+1
        if(y(i).le.y(ilo)) ilo=i
        if(y(i).gt.y(ihi)) then
          inhi=ihi
          ihi=i
        else if(y(i).gt.y(inhi)) then
          if(i.ne.ihi) inhi=i
        endif
13    continue
c      rtol=2.*abs(y(ihi)-y(ilo))/(abs(y(ihi))+abs(y(ilo))+TINY)
      if (itol.ge.ntol) then
c       write(6,'(i2,2(a8,e12.6))')
c     *                   itol,'   rtol=',rtol,' y(ihi)=',y(ihi)
        swap=y(1)
        y(1)=y(ilo)
        y(ilo)=swap
        do 14 n=1,ndim
          swap=p(1,n)
          p(1,n)=p(ilo,n)
          p(ilo,n)=swap
14      continue
        return
      endif
      if (iter.ge.ITMAX) pause 'ITMAX exceeded in amoeba'
      iter=iter+2
      ytry=amotry(p,y,psum,mp,mp0,np,np0,ndim,funk,ihi,-1.)
      if (ytry.le.y(ilo)) then
        ytry=amotry(p,y,psum,mp,mp0,np,np0,ndim,funk,ihi,2.)
      else if (ytry.ge.y(inhi)) then
        ysave=y(ihi)
        ytry=amotry(p,y,psum,mp,mp0,np,np0,ndim,funk,ihi,0.5)
        if (ytry.ge.ysave) then
          do 16 i=1,ndim+1
            if(i.ne.ilo)then
              do 15 j=1,ndim
                psum(j)=0.5*(p(i,j)+p(ilo,j))
               p(i,j)=psum(j)
15            continue
              y(i)=funk(psum)
            endif
16        continue
          iter=iter+ndim
          goto 1
        endif
      else
        iter=iter-1
      endif
      itol=itol+1
      goto 2
      END
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      FUNCTION amotry(p,y,psum,mp,mp0,np,np0,ndim,funk,ihi,fac)
c Numerical Recipe', modified for simulated anealing approach by T. Yokoi 
      INTEGER ihi,mp,ndim,np,NMAX
      REAL amotry,fac,p(mp0,np0),psum(np0),y(mp0),funk
      PARAMETER (NMAX=50,lmax=25,lmax2=50,nf0=512,big=1.e+30)
      EXTERNAL RAYMRX,funk
CU    USES funk
      INTEGER j,n_roh,n_vp,IL,itr,ia,nf,nn
      REAL fac1,fac2,ytry,ptry(NMAX),tk,tol
      real pmn(lmax2),pmx(lmax2),h(lmax),roh(lmax),vp(lmax),vs(lmax),
     *     fr(nf0),sd_obs(nf0),vr_obs(nf0),y0(4),yij(42),
     *     ap(lmax),ae(lmax)
      COMMON /Temp/tk /Range/pmn,pmx 
      COMMON /Layer/h,roh,vp,vs,IL
      COMMON /Flag/n_roh,n_vp
      COMMON /Obs/nf,fr,vr_obs,sd_obs
      COMMON /Crtl/ap,ae,y0,yij,tol,itr,ia
      COMMON /Seed/idum
c      write(6,*)' amotry'
      nn=0
   10 fac1=(1.-fac)/real(ndim)
      fac2=fac1-fac
      do j=1,ndim
       ptry(j)=psum(j)*fac1-p(ihi,j)*fac2
       if(abs(fac).ge.1.) then
        rn1=ran1(idum)
        aa=(rn1-0.5)/abs(rn1-0.5)*tk*((1.+1./tk)**abs(2.*rn1-1.)-1.)
        ptry(j)=ptry(j)+(pmx(j)-pmn(j))*aa
       endif
      enddo
      do j=1,ndim
       if(ptry(j).lt.pmn(j).or.ptry(j).gt.pmx(j)) then
        if(nn.ge.500) then
	 goto 11
	endif
	nn=nn+1
	goto 10
       endif
      enddo
      goto 12
   11 ytry=big
      goto 13
   12 ytry=funk(ptry)
   13 continue
      if (ytry.lt.y(ihi)) then
        y(ihi)=ytry
        do 22 j=1,ndim
          psum(j)=psum(j)-p(ihi,j)+ptry(j)
          p(ihi,j)=ptry(j)
   22   continue
c       write(6,*)'100 0 Down hill'
c       write(6,'(8f10.4)')(ptry(j),j=1,ndim)
c        write(6,'(f16.12)')ytry
      else
       if(ytry.lt.big/2.) then
        de=ytry-y(ihi)
        if(ran1(idum).lt.exp(-de/tk)) then
         y(ihi)=ytry
         do 23 j=1,ndim
          psum(j)=psum(j)-p(ihi,j)+ptry(j)
          p(ihi,j)=ptry(j)
   23    continue
c         write(6,*)'010 1 Up accepted'
c         write(6,'(8f10.4)')(ptry(j),j=1,ndim)
c        write(6,'(f16.12)')ytry
        else
c	 write(6,*)'001 2 Up rejected'
	endif
       endif
      endif
c      write(6,'(f16.12)')y(ihi)
      amotry=ytry
      return
      END
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      function ran1(idum)
      implicit none
      integer idum
      real ran1
      integer::i
      double precision::d1  
      call pre_random
      call random_number(d1)
      ran1=d1
c     write(6,*)'Random number=',ran1
      return
      end
c
      subroutine pre_random
      implicit none
      integer::seedsize,c
      integer,allocatable::seed(:)
      seedsize=1
      call random_seed(size=seedsize)
      allocate(seed(1:seedsize))
      call system_clock(count=c)
      seed=c
      call random_seed(put=seed)
      return
      end
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE  RAYDSP(DIFFEQ,H,RHO,VP,VS,AP,AE,L,                    RAY02770
     *                   W,CMN,CMX,DC,TOL,ITR,IA,C,U,EK,                RAY02780
     1                   Y0,YIJ,IER)                                    RAY02790
C Seismological Algorithm
C                                                                       RAY02800
C     INPUT                                                             RAY02810
C       DIFFEQ : SUBROUTINE TO INTEGRATE EQ. OF MOTION                  RAY02820
C       H    : LAYER THICKNESS.  H(L) IS ARBITRARY                      RAY02830
C       RHO  : DENSITY                                                  RAY02840
C       VP   : COMPRESSIONAL WAVE VELOCITY                              RAY02850
C       VS   : SHEAR WAVE VELOCITY                                      RAY02860
C       AP   : ANISOTROPY FACTOR PHI                                    RAY02870
C       AE   : ANISOTROPY FACTOR ETA                                    RAY02880
C       L    : NUMBER OF LAYERS INCLUDING THE BOTTOM                    RAY02890
C              HALF-SPACE                                               RAY02900
C       W    : ANGULAR FREQUENCY                                        RAY02910
C       CMN  : LOWER LIMIT OF PHASE VELOCITY                            RAY02920
C       CMX  : UPPER LIMIT OF PHASE VELOCITY                            RAY02930
C       DC   : INCREMENT   OF PHASE VELOCITY                            RAY02940
C       TOL  : RELATIVE ACCURACY OF PHASE VELOCITY                      RAY02950
C       ITR  : MAXIMUM NUMBER OF ITERATIONS                             RAY02960
C       IA   : = 0 ;   ISOTROPIC MODEL                                  RAY02970
C              = 1 ; ANISOTROPIC MODEL                                  RAY02980
C     OUTPUT                                                            RAY02990
C       C    : PHASE VELOCITY                                           RAY03000
C       U    : GROUP VELOCITY BY DIFFERENTIATION                        RAY03010
C       EKD  : ENERGY INTEGRAL BY DIFFERENTIATION                       RAY03020
C              2*K**2*I3                                                RAY03030
C       Y0   : SURFACE VALUES OF EIGENFUNCTION                          RAY03040
C              Y0(1) ; Y1 (SCALE FACTOR)                                RAY03050
C              Y0(2) ; Y2/ABS(Y1), DISPERSION FUNCTION                  RAY03060
C              Y0(3) ; Y3/Y1                                            RAY03070
C       YIJ  : SURFACE VALUES OF YIJ (COMPOUNDS),                       RAY03080
C              C*DYIJ/DC, AND W*DYIJ/DW                                 RAY03090
C       IER  : RETURN CODE                                              RAY03100
C              < 0 ; INPUT ERROR                                        RAY03110
C              = 0 ; NO ERROR                                           RAY03120
C              = 1 ; SLOW CONVERGENCE                                   RAY03130
C              = 2 ; ROOT NOT FOUND                                     RAY03140
C                                                                       RAY03150
C     SUBROUTINE : DIFFEQ (RAYMRX OR RAYRKG)                            RAY03160
C                                                                       RAY03170
C     DISPER-80,  VER-86                                                RAY03180
C                                                                       RAY03190
C     M. SAITO  23/VI/79                                                RAY03200
C     REVISED   25/IX/85                                                RAY03210
C                                                                       RAY03220
      DIMENSION  H(L),RHO(L),VP(L),VS(L),AP(L),AE(L),                   RAY03230
     *           Y0(3),YIJ(15)                                          RAY03240
C                                                                       RAY03250
C     INITIALIZATION                                                    RAY03260
C                                                                       RAY03270
      IF( L.LE.2 .OR. W.LE.0 .OR. DC.EQ.0 )  GO TO  90                  RAY03280
      ONE = 1                                                           RAY03290
C                                                                       RAY03300
C     MACHINE EPSILON                                                   RAY03310
C                                                                       RAY03320
      EPS = 1                                                           RAY03330
    1 IF( (1+EPS).LE.1 )  GO TO  2                                      RAY03340
        EPS = EPS/2                                                     RAY03350
        GO TO  1                                                        RAY03360
    2 EPS = EPS/2                                                       RAY03370
c      write(6,*) eps
C                                                                       RAY03380
      TOL1 = TOL                                                        RAY03390
      IF( TOL1.GT.0 )  TOL1 = MAX(TOL1,EPS)                             RAY03400
      IER1 = 0                                                          RAY03410
cyokoi      WRITE(6,3)  W
    3   FORMAT(/7X,'W',15X,'RAYLEIGH WAVE'/                             RAY03430
     *         1PE18.6//7X,'C',17X,'Y2',16X,'Y3')                       RAY03440
C    *         1PD18.6//7X,'C',17X,'Y2',16X,'Y3')                       RAY03450
      C3 = CMN                                                          RAY03460
      IF( C3.LE.0 )  GO TO  92                                          RAY03470
      CALL  DIFFEQ(H,RHO,VP,VS,AP,AE,L,W,C3,IA,1,U,                     RAY03480
     *             EK,Y0,YIJ,IER)                                       RAY03490
      IF( IER.NE.0 )  RETURN                                            RAY03500
      F3 = Y0(2)                                                        RAY03510
cyokoi      WRITE(6,4)  C3,F3,Y0(3)
    4   FORMAT(1P3E18.6)                                                RAY03530
C   4   FORMAT(1P3D18.6)                                                RAY03540
      IF( F3.EQ.0 .AND. TOL1.GT.0 )  GO TO  9                           RAY03550
C                                                                       RAY03560
C     FIND A ZERO-CROSS                                                 RAY03570
C                                                                       RAY03580
      KX = (CMX - CMN)/DC + 0.5                                         RAY03590
      KX = MAX(1,KX)                                                    RAY03600
      DO  5  K=1,KX                                                     RAY03610
        CC = CMN + K*DC                                                 RAY03620
        C1 = C3                                                         RAY03630
        F1 = F3                                                         RAY03640
        C3 = CC                                                         RAY03650
        IF( C3.LE.0 )  GO TO  92                                        RAY03660
        CALL  DIFFEQ(H,RHO,VP,VS,AP,AE,L,W,C3,IA,1,U,                   RAY03670
     *               EK,Y0,YIJ,IER)                                     RAY03680
        IF( IER.NE.0 )  RETURN                                          RAY03690
        F3 = Y0(2)                                                      RAY03700
cyokoi        WRITE(6,4)  C3,F3,Y0(3)
        IF(  TOL1.LE.0 )  GO TO  5                                      RAY03720
        IF( F3*SIGN(ONE,F1).LE.0 )  GO TO  6                            RAY03730
    5 CONTINUE                                                          RAY03740
C                                                                       RAY03750
      IER = 2                                                           RAY03760
      IF( TOL1.LE.0 )  RETURN                                           RAY03770
      GO TO  94                                                         RAY03780
C                                                                       RAY03790
C     INTERPOLATION                                                     RAY03800
C                                                                       RAY03810
    6 IF( F3.EQ.0 )  GO TO  9                                           RAY03820
      C2 = C3                                                           RAY03830
      F2 = F3                                                           RAY03840
      E  = C1 - C2                                                      RAY03850
      D  = E/2                                                          RAY03860
      C3 = C2 + D                                                       RAY03870
      KX = MAX(1,ITR)                                                   RAY03880
C                                                                       RAY03890
      DO  7  K=1,KX                                                     RAY03900
        CALL  DIFFEQ(H,RHO,VP,VS,AP,AE,L,W,C3,IA,1,U,                   RAY03910
     *               EK,Y0,YIJ,IER)                                     RAY03920
        F3 = Y0(2)                                                      RAY03930
cyokoi        WRITE(6,4) C3,F3,Y0(3)
        IF( F3*SIGN(ONE,F2).GT.0 )  THEN                                RAY03950
          FF = C1                                                       RAY03960
          C1 = C2                                                       RAY03970
          C2 = FF                                                       RAY03980
          FF = F1                                                       RAY03990
          F1 = F2                                                       RAY04000
          F2 = FF                                                       RAY04010
        END IF                                                          RAY04020
        IF( ABS(F3).GT.ABS(F2) )  THEN                                  RAY04030
          FF = C2                                                       RAY04040
          C2 = C3                                                       RAY04050
          C3 = FF                                                       RAY04060
          FF = F2                                                       RAY04070
          F2 = F3                                                       RAY04080
          F3 = FF                                                       RAY04090
        END IF                                                          RAY04100
        E = C2 - C3                                                     RAY04110
        IF( F3.EQ.0 )  GO TO  9                                         RAY04120
        TOLC = C3*TOL1                                                  RAY04130
        DD = D                                                          RAY04140
        F32 = F3/F2                                                     RAY04150
        F31 = F3/F1                                                     RAY04160
        F21 = F2/F1                                                     RAY04170
        Q = F32*(E*(1 - F31) + F21*(F31 - F21)*(C1 - C3))               RAY04180
        S = (F21 - 1)*(F32 - 1)*(F31 - 1)                               RAY04190
C                                                                       RAY04200
C       TEST RANGE                                                      RAY04210
C                                                                       RAY04220
        IF( Q.LT.0 )  S =-S                                             RAY04230
        Q = ABS(Q)                                                      RAY04240
        IF( Q.GE.(E*S-ABS(TOLC*S)) )  THEN                              RAY04250
C                                                                       RAY04260
C       LINEAR INTERPOLATION                                            RAY04270
C                                                                       RAY04280
          D = E*F32/(F32 - 1)                                           RAY04290
C                                                                       RAY04300
C       INVERSE QUADRATIC INTERPOLATION                                 RAY04310
C                                                                       RAY04320
        ELSE                                                            RAY04330
          D = Q/S                                                       RAY04340
        END IF                                                          RAY04350
C                                                                       RAY04360
C       TEST CONVERGENCE                                                RAY04370
C                                                                       RAY04380
        C1 = C2                                                         RAY04390
        F1 = F2                                                         RAY04400
        C2 = C3                                                         RAY04410
        F2 = F3                                                         RAY04420
        C3 = C2 + D                                                     RAY04430
        IF( ABS(E).LE.TOLC )  GO TO  9                                  RAY04440
        IF( ABS(D).LE.TOLC )  THEN                                      RAY04450
C                                                                       RAY04460
C       BISECTION                                                       RAY04470
C                                                                       RAY04480
          IF( ABS(DD).LE.TOLC )  THEN                                   RAY04490
            D = E/2                                                     RAY04500
            C3 = C2 + D                                                 RAY04510
          ELSE                                                          RAY04520
            C3 = C2 + SIGN(TOLC,D)                                      RAY04530
          END IF                                                        RAY04540
        END IF                                                          RAY04550
C                                                                       RAY04560
    7 CONTINUE                                                          RAY04570
C                                                                       RAY04580
C     SLOW CONVERGENCE                                                  RAY04590
C                                                                       RAY04600
      WRITE(6,8)  KX                                                    RAY04610
    8   FORMAT(20X,5('?'),3X,'(RAYDSP)   SLOW CONV. ',                  RAY04620
     *         'AFTER',I5,' ITERATIONS',3X,5('?'))                      RAY04630
      IER = 1                                                           RAY04640
C                                                                       RAY04650
C     ROOT IS FOUND                                                     RAY04660
C                                                                       RAY04670
    9 CALL  DIFFEQ(H,RHO,VP,VS,AP,AE,L,W,C3,IA,3,U,                     RAY04680
     *             EK,Y0,YIJ,IER1)                                      RAY04690
cyokoi      WRITE(6,4)  C3,Y0(2),Y0(3)
      C   = C3                                                          RAY04710
      RETURN                                                            RAY04720
C                                                                       RAY04730
C     INPUT ERROR                                                       RAY04740
C                                                                       RAY04750
   90 WRITE(6,91)  L,W,DC                                               RAY04760
   91   FORMAT(20X,5('?'),3X,'(RAYDSP)   INPUT ERROR',3X,               RAY04770
     *         'L  =',I5,3X,'W  =',1PE13.6,3X,                          RAY04780
     1         'DC =',E13.6,3X,5('?'))                                  RAY04790
      IER =-1                                                           RAY04800
      RETURN                                                            RAY04810
C                                                                       RAY04820
   92 WRITE(6,93)  C3                                                   RAY04830
   93   FORMAT(20X,5('?'),3X,'(RAYDSP)   INPUT ERROR',3X,               RAY04840
     *         'C  =',1PE13.6,3X,5('?'))                                RAY04850
      IER =-1                                                           RAY04860
      RETURN                                                            RAY04870
C                                                                       RAY04880
C     NO ROOT                                                           RAY04890
C                                                                       RAY04900
   94 WRITE(6,95)                                                       RAY04910
   95   FORMAT(20X,5('?'),3X,'(RAYDSP)',3X,                             RAY04920
     *         'ROOT NOT FOUND',3X,5('?'))                              RAY04930
      RETURN                                                            RAY04940
      END                                                               RAY04950
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C                                                                       RAY08420
C     RAYLEIGH WAVE MATRIX METHOD INTEGRATION                           RAY08430
C                                                                       RAY08440
      SUBROUTINE  RAYMRX(H,RHO,VP,VS,AP,AE,L,W,C,                       RAY08450
     *                   IA,IG,U,EK,Y0,Y,IER)                           RAY08460
C                                                                       RAY08470
C     INPUT                                                             RAY08480
C       H    : LAYER THICKNESS.  H(L) IS ARBITRARY                      RAY08490
C       RHO  : DENSITY                                                  RAY08500
C       VP   : P WAVE VELOCITY                                          RAY08510
C       VS   : S WAVE VELOCITY                                          RAY08520
C       L    : NUMBER OF LAYERS INCLDING THE BOTTOM                     RAY08530
C              HALF-SPACE                                               RAY08540
C       W    : ANGULAR FREQUENCY                                        RAY08550
C       C    : PHASE VELOCITY                                           RAY08560
C       IG   : = 1 ; TO COMPUTE Y ONLY                                  RAY08570
C              = 2 ; TO COMPUTE Y AND C*CY/DC                           RAY08580
C              = 3 ; TO COMPUTE Y, C*DY/DC, AND W*DY/DW                 RAY08590
C     OUTPUT                                                            RAY08600
C       U    : GROUP VELOCITY BY DIFFERENTIATION (IG=3)                 RAY08610
C       EK   : ENERGY INTEGRAL 2*K**2*I3  (IG>=2)                       RAY08620
C       Y0   : SURFACE VALUES OF EIGENFUNCTIONS                         RAY08630
C              Y0(1) ; Y1 (SCALE FACTOR)                                RAY08640
C              Y0(2) ; Y2/ABS(Y1), DISPERSION FUNCTION                  RAY08650
C              Y0(3) ; Y3/Y1                                            RAY08660
C       Y    : SURFACE VALUES OF YIJ (COMPOUNDS),                       RAY08670
C              C*DYIJ/DC, AND W*DYIJ/DW                                 RAY08680
C              FOR SOLID LAYER                                          RAY08690
C                (IJ) = (12),(13),(14),(23),(24)                        RAY08700
C       IER  : RETURN CODE                                              RAY08710
C              < 0 ; INPUT ERROR                                        RAY08720
C              = 0 ; NO ERROR                                           RAY08730
C                                                                       RAY08740
C     ISOTROPIC MODEL ONLY.  AP, AE, IA ARE DUMMY.                      RAY08750
C                                                                       RAY08760
C     DISPER-80,  VER-86                                                RAY08770
C                                                                       RAY08780
C     M. SAITO  30/VII/79                                               RAY08790
C     REVISED   10/XII/86                                               RAY08800
C                                                                       RAY08810
      DIMENSION  H(L),RHO(L),VP(L),VS(L),Y0(3),Y(15),Z(15)              RAY08820
     *          ,AP(1),AE(1)                                            RAY08830
      DATA  EPS/1.E-10/,BIG/1.E+10/                                     RAY08840
C                                                                       RAY08850
C     DEFINE SINH(X)/X AND (COSH(X)-SINH(X)/X)/X**2                     RAY08860
C                                                                       RAY08870
      SH0(X) = 0.9999997 + X*(0.1666667 + X*(0.0083361                  RAY08880
     *       + X*0.0001984))                                            RAY08890
      SH1(X) = 0.3333333 + X*(0.0333333 + X*(0.0011907                  RAY08900
     *       + X*0.0000220))                                            RAY08910
C                                                                       RAY08920
C     FOR DOUBLE PRECISION USE                                          RAY08930
C                                                                       RAY08940
C     SH0(X) = 1.0D0 + X*(1.6666 66666 66666 7D-1                       RAY08950
C    *       + X*(8.33 33333 33334 0D-3                                 RAY08960
C    1       + X*(1.9 84126 98412 7D-4                                  RAY08970
C    2       + X*(2.7557 31918 9D-6 + X*(2.50 12108 4D-8                RAY08980
C    3       + X*(1.60596 1D-10 + X*7.64 7D-13))))))                    RAY08990
C     SH1(X) = 3.3333 33333 33333 3D-1                                  RAY09000
C    *       + X*(3.333 33333 33333 3D-2                                RAY09010
C    1       + X*(1.19 04761 90476 2D-3                                 RAY09020
C    2       + X*(2.20458 55379 2D-5                                    RAY09030
C    3       + X*(2.505 21083 7D-7 + X*(1.9 27085 3D-9                  RAY09040
C    4       + X*(1.0706 3D-11 + X*4.50D-14))))))                       RAY09050
C                                                                       RAY09060
C     INITIAL VALUE                                                     RAY09070
C                                                                       RAY09080
      IF( L.LE.0 .OR. W.LE.0 .OR. C.LE.0 )  GO TO  90                   RAY09090
      I   = L                                                           RAY09100
      RO  = RHO(I)                                                      RAY09110
      IF( RO.LE.0..OR. C.GE.VP(I) )  GO TO  92                          RAY09120
      IER = 0                                                           RAY09130
      CC  = C*C                                                         RAY09140
      WN  = W/C                                                         RAY09150
      IGG = MAX(1,MIN(3,IG))                                            RAY09160
      ROC = RO*CC                                                       RAY09170
      SV  = VS(I)                                                       RAY09180
      CP  = C/VP(I)                                                     RAY09190
      RAA = (1 + CP)*(1 - CP)                                           RAY09200
      RA  = SQRT(RAA)                                                   RAY09210
      DO  1  J=1,15                                                     RAY09220
    1   Y(J) = 0                                                        RAY09230
C                                                                       RAY09240
C     LIQUID BOTTOM                                                     RAY09250
C                                                                       RAY09260
      IF( SV.LE.0 )  THEN                                               RAY09270
        Y(1) = RA*EPS                                                   RAY09280
        Y(2) =-ROC*EPS                                                  RAY09290
        JX = 2                                                          RAY09300
C                                                                       RAY09310
        IF( IGG.GE.2 )  THEN                                            RAY09320
          Y(3) =-CP**2*EPS/RA                                           RAY09330
          Y(4) =-2*ROC*EPS                                              RAY09340
          JX = 4                                                        RAY09350
C                                                                       RAY09360
          IF( IGG.GT.2 )  JX = 6                                        RAY09370
        ENDIF                                                           RAY09380
C                                                                       RAY09390
C     SOLID BOTTOM                                                      RAY09400
C                                                                       RAY09410
      ELSE                                                              RAY09420
        IF( C.GE.SV )  GO TO  92                                        RAY09430
        CS  = C/SV                                                      RAY09440
        RBB = (1 + CS)*(1 - CS)                                         RAY09450
        RB  = SQRT(RBB)                                                 RAY09460
        RG  = 2*RO*VS(I)**2                                             RAY09470
        Y(3) =-RA*EPS                                                   RAY09480
        Y(4) =-RB*EPS                                                   RAY09490
        Y(2) =-EPS*(CP**2*RBB + CS**2)/(ROC*(RA*RB + 1))                RAY09500
        Y(1) = RG*Y(2) + EPS                                            RAY09510
        Y(5) =-RG*(Y(1) + EPS) + ROC*EPS                                RAY09520
        JX = 5                                                          RAY09530
C                                                                       RAY09540
        IF( IGG.GE.2 )  THEN                                            RAY09550
          Y(8) = EPS*CP**2/RA                                           RAY09560
          Y(9) = EPS*CS**2/RB                                           RAY09570
          Y(7) =-(RB*Y(8) + RA*Y(9))/ROC - 2*Y(2)                       RAY09580
          Y(6) = RG*Y(7)                                                RAY09590
          Y(10)=-RG*Y(6) + EPS*ROC*2                                    RAY09600
          JX = 10                                                       RAY09610
C                                                                       RAY09620
          IF( IGG.GT.2 )  JX = 15                                       RAY09630
        ENDIF                                                           RAY09640
      ENDIF                                                             RAY09650
C                                                                       RAY09660
C     INTEGRATE UPWARD                                                  RAY09670
C                                                                       RAY09680
      IF( L.LE.1 )  GO TO  8                                            RAY09690
      DO  7  II=2,L                                                     RAY09700
        I  = I - 1                                                      RAY09710
        RO = RHO(I)                                                     RAY09720
        ROC= RO*CC                                                      RAY09730
        PV = VP(I)                                                      RAY09740
        SV = VS(I)                                                      RAY09750
        IF( PV.LE.0 .OR. RO.LE.0 .OR.                                   RAY09760
     *      H(I).LE.0 )  GO TO  92                                      RAY09770
        DO  2  J=1,15                                                   RAY09780
    2     Z(J) = Y(J)                                                   RAY09790
        IF( (SV.LE.0 .AND. VS(I+1).LE.0) .OR.                           RAY09800
     *      (SV.GT.0 .AND. VS(I+1).GT.0) )  GO TO  3                    RAY09810
C                                                                       RAY09820
C       SOLID  -----> LIQUID BOUNDARY                                   RAY09830
C                                                                       RAY09840
        IF( SV.LE.0 )  THEN                                             RAY09850
          Y0(3) =-Y(1)/Y(3)                                             RAY09860
          Z(1) = Y(3)                                                   RAY09870
          Z(2) = Y(5)                                                   RAY09880
          JX = 2                                                        RAY09890
C                                                                       RAY09900
          IF( IGG.GE.2 )  THEN                                          RAY09910
            Z(3) = Y(8)                                                 RAY09920
            Z(4) = Y(10)                                                RAY09930
            JX = 4                                                      RAY09940
C                                                                       RAY09950
            IF( IGG.GT.2 )  THEN                                        RAY09960
              Z(5) = Y(13)                                              RAY09970
              Z(6) = Y(15)                                              RAY09980
              JX = 6                                                    RAY09990
            ENDIF                                                       RAY10000
          ENDIF                                                         RAY10010
C                                                                       RAY10020
C       LIQUID -----> SOLID BOUNDARY                                    RAY10030
C                                                                       RAY10040
        ELSE                                                            RAY10050
          Z( 2) = Y(1)                                                  RAY10060
          Z( 4) = Y(2)                                                  RAY10070
          Z( 1) = 0                                                     RAY10080
          Z( 3) = 0                                                     RAY10090
          Z( 5) = 0                                                     RAY10100
          JX = 5                                                        RAY10110
C                                                                       RAY10120
          IF( IGG.GE.2 )  THEN                                          RAY10130
            Z( 7) = Y(3)                                                RAY10140
            Z( 9) = Y(4)                                                RAY10150
            Z( 6) = 0                                                   RAY10160
            Z( 8) = 0                                                   RAY10170
            Z(10) = 0                                                   RAY10180
            JX = 10                                                     RAY10190
C                                                                       RAY10200
            IF( IGG.GT.2 )  THEN                                        RAY10210
              Z(12) = Y(5)                                              RAY10220
              Z(14) = Y(6)                                              RAY10230
              Z(11) = 0                                                 RAY10240
              Z(13) = 0                                                 RAY10250
              Z(15) = 0                                                 RAY10260
              JX = 15                                                   RAY10270
            ENDIF                                                       RAY10280
          ENDIF                                                         RAY10290
        ENDIF                                                           RAY10300
C                                                                       RAY10310
    3   R2  = 1/ROC                                                     RAY10320
        CP  = C/PV                                                      RAY10330
        RAA = (1 + CP)*(1 - CP)                                         RAY10340
        CS  = 0                                                         RAY10350
        IF( SV.GT.0 )  CS = C/SV                                        RAY10360
        RBB = (1 + CS)*(1 - CS)                                         RAY10370
        HK  = H(I)*WN                                                   RAY10380
        HKK = HK**2                                                     RAY10390
        XX  = RAA*HKK                                                   RAY10400
        ONE = 1                                                         RAY10410
C                                                                       RAY10420
C       SINH(X)/X                                                       RAY10430
C                                                                       RAY10440
        DO  4  K=1,2                                                    RAY10450
          CHA = CHB                                                     RAY10460
          SHA = SHB                                                     RAY10470
          DHA = DHB                                                     RAY10480
          AA  = ABS(XX)                                                 RAY10490
          IF( AA.LE.1 )  THEN                                           RAY10500
            SHB = SH0(XX)                                               RAY10510
            CHB = 1 + XX*SH0(XX/4)**2/2                                 RAY10520
            IF( IGG.GE.2 )  DHB = SH1(XX)*HKK                           RAY10530
          ELSE                                                          RAY10540
            AA  = SQRT(AA)                                              RAY10550
            IF( XX.LE.0 )  THEN                                         RAY10560
              CHB = COS(AA)                                             RAY10570
              SHB = SIN(AA)/AA                                          RAY10580
            ELSE                                                        RAY10590
              IF( AA.GT.100 )  ONE = 0                                  RAY10600
              IF( AA.LE.100 )  ONE = ONE/COSH(AA)                       RAY10610
              CHB = 1                                                   RAY10620
              SHB = TANH(AA)/AA                                         RAY10630
            ENDIF                                                       RAY10640
            IF( IGG.GE.2 )  DHB = (HKK/XX)*(CHB - SHB)                  RAY10650
          ENDIF                                                         RAY10660
          XX  = HKK*RBB                                                 RAY10670
          SHB = HK*SHB                                                  RAY10680
    4   CONTINUE                                                        RAY10690
C                                                                       RAY10700
C     LAYER MATRICES                                                    RAY10710
C                                                                       RAY10720
C       LIQUID LAYER                                                    RAY10730
C                                                                       RAY10740
        IF( SV.LE.0 )  THEN                                             RAY10750
          B11 = CHA                                                     RAY10760
          B12 =-RAA*SHA*R2                                              RAY10770
          B21 =-ROC*SHA                                                 RAY10780
C                                                                       RAY10790
          Y(1) = B11*Z(1) + B12*Z(2)                                    RAY10800
          Y(2) = B21*Z(1) + B11*Z(2)                                    RAY10810
C                                                                       RAY10820
          IF( IGG.GE.2 )  THEN                                          RAY10830
            C11 =-HK*SHA                                                RAY10840
            C12 = (HK*CHA + (1 + RAA)*SHA)*R2                           RAY10850
            C21 = (HK*DHA - SHA)*ROC                                    RAY10860
C                                                                       RAY10870
            Y(3) = B11*Z(3) + B12*Z(4) + C11*Z(1)                       RAY10880
     *           + C12*Z(2)                                             RAY10890
            Y(4) = B21*Z(3) + B11*Z(4) + C21*Z(1)                       RAY10900
     *           + C11*Z(2)                                             RAY10910
C                                                                       RAY10920
            IF( IGG.GT.2 )  THEN                                        RAY10930
              W11 = HK*RAA*SHA                                          RAY10940
              W12 =-HK*RAA*CHA*R2                                       RAY10950
              W21 =-HK*ROC*CHA                                          RAY10960
C                                                                       RAY10970
              Y(5) = B11*Z(5) + B12*Z(6) + W11*Z(1)                     RAY10980
     *             + W12*Z(2)                                           RAY10990
              Y(6) = B21*Z(5) + B11*Z(6) + W21*Z(1)                     RAY11000
     *             + W11*Z(2)                                           RAY11010
            ENDIF                                                       RAY11020
          ENDIF                                                         RAY11030
C                                                                       RAY11040
C       SOLID LAYER                                                     RAY11050
C                                                                       RAY11060
        ELSE                                                            RAY11070
          G1  = 2/CS**2                                                 RAY11080
          RG  = G1*ROC                                                  RAY11090
          R4  = RG - ROC                                                RAY11100
          E1  = CHA*CHB                                                 RAY11110
          E2  = E1 - ONE                                                RAY11120
          E3  = SHA*SHB                                                 RAY11130
          E5  = SHA*CHB                                                 RAY11140
          E6  = SHB*CHA                                                 RAY11150
          F1  = E2 - E3                                                 RAY11160
          F2  = R2*F1                                                   RAY11170
          F3  = G1*F1 + E3                                              RAY11180
          B33 = E1                                                      RAY11190
          B34 = RAA*E3                                                  RAY11200
          B43 = RBB*E3                                                  RAY11210
          B25 =-R2*(F2 + R2*(E2 - RAA*B43))                             RAY11220
          B15 = RG*B25 + F2                                             RAY11230
          B16 =-RG*B15 - F3                                             RAY11240
          B22 = B16 + E1                                                RAY11250
          B12 = RG*B16 - R4*F3                                          RAY11260
          B52 =-RG*B12 + R4*(RG*F3 + R4*E3)                             RAY11270
          B23 = R2*(E5 - RBB*E6)                                        RAY11280
          B13 = RG*B23 - E5                                             RAY11290
          B42 =-RG*B13 + R4*E5                                          RAY11300
          B24 = R2*(E6 - RAA*E5)                                        RAY11310
          B14 = RG*B24 - E6                                             RAY11320
          B32 =-RG*B14 + R4*E6                                          RAY11330
          B11 = ONE - B16-B16                                           RAY11340
          B21 = B15+B15                                                 RAY11350
          B31 = B14+B14                                                 RAY11360
          B41 = B13+B13                                                 RAY11370
          B51 = B12+B12                                                 RAY11380
C                                                                       RAY11390
          Y(1) = B11*Z(1) + B12*Z(2) + B13*Z(3)                         RAY11400
     *         + B14*Z(4) + B15*Z(5)                                    RAY11410
          Y(2) = B21*Z(1) + B22*Z(2) + B23*Z(3)                         RAY11420
     *         + B24*Z(4) + B25*Z(5)                                    RAY11430
          Y(3) = B31*Z(1) + B32*Z(2) + B33*Z(3)                         RAY11440
     *         + B34*Z(4) + B24*Z(5)                                    RAY11450
          Y(4) = B41*Z(1) + B42*Z(2) + B43*Z(3)                         RAY11460
     *         + B33*Z(4) + B23*Z(5)                                    RAY11470
          Y(5) = B51*Z(1) + B52*Z(2) + B42*Z(3)                         RAY11480
     *         + B32*Z(4) + B22*Z(5)                                    RAY11490
C                                                                       RAY11500
          IF( IGG.GE.2 )  THEN                                          RAY11510
            RAAC =-2*CP*CP                                              RAY11520
            RBBC =-2*CS*CS                                              RAY11530
            R1C = ROC+ROC                                               RAY11540
            E1C =-HK*(E5 + E6)                                          RAY11550
            E3C =-E3-E3 - HK*(DHA*SHB + DHB*SHA)                        RAY11560
            E5C =-E5 - HK*(DHA*CHB + E3)                                RAY11570
            E6C =-E6 - HK*(DHB*CHA + E3)                                RAY11580
            F1C = E1C - E3C                                             RAY11590
            F2C = R2*(F1C - F1-F1)                                      RAY11600
            F3C = G1*(F1C - F1-F1) + E3C                                RAY11610
            C33 = E1C                                                   RAY11620
            C34 = RAA*E3C + RAAC*E3                                     RAY11630
            C43 = RBB*E3C + RBBC*E3                                     RAY11640
            C25 =-R2*(F2C + R2*(E1C - RAA*C43 - RAAC*B43))              RAY11650
     *          - 2*(B25+B25 + R2*F2)                                   RAY11660
            C15 = RG*C25 + F2C                                          RAY11670
            C16 =-RG*C15 - F3C                                          RAY11680
            C22 = C16 + E1C                                             RAY11690
            C12 = RG*C16 + R1C*F3 - R4*F3C                              RAY11700
            C52 =-RG*C12 + R4*(RG*F3C + R4*E3C)                         RAY11710
     *          - R1C*(RG*F3 + 2*R4*E3)                                 RAY11720
            C23 = R2*(E5C - RBB*E6C - RBBC*E6) - B23-B23                RAY11730
            C13 = RG*C23 - E5C                                          RAY11740
            C42 =-RG*C13 + R4*E5C - R1C*E5                              RAY11750
            C24 = R2*(E6C - RAA*E5C - RAAC*E5) - B24-B24                RAY11760
            C14 = RG*C24 - E6C                                          RAY11770
            C32 =-RG*C14 + R4*E6C - R1C*E6                              RAY11780
            C11 =-C16-C16                                               RAY11790
            C21 = C15+C15                                               RAY11800
            C31 = C14+C14                                               RAY11810
            C41 = C13+C13                                               RAY11820
            C51 = C12+C12                                               RAY11830
C                                                                       RAY11840
            Y( 6) = B11*Z(6) + B12*Z(7) + B13*Z(8)                      RAY11850
     *            + B14*Z(9) + B15*Z(10)                                RAY11860
     *            + C11*Z(1) + C12*Z(2) + C13*Z(3)                      RAY11870
     *            + C14*Z(4) + C15*Z(5)                                 RAY11880
            Y( 7) = B21*Z(6) + B22*Z(7) + B23*Z(8)                      RAY11890
     *            + B24*Z(9) + B25*Z(10)                                RAY11900
     *            + C21*Z(1) + C22*Z(2) + C23*Z(3)                      RAY11910
     *            + C24*Z(4) + C25*Z(5)                                 RAY11920
            Y( 8) = B31*Z(6) + B32*Z(7) + B33*Z(8)                      RAY11930
     *            + B34*Z(9) + B24*Z(10)                                RAY11940
     *            + C31*Z(1) + C32*Z(2) + C33*Z(3)                      RAY11950
     *            + C34*Z(4) + C24*Z(5)                                 RAY11960
            Y( 9) = B41*Z(6) + B42*Z(7) + B43*Z(8)                      RAY11970
     *            + B33*Z(9) + B23*Z(10)                                RAY11980
     *            + C41*Z(1) + C42*Z(2) + C43*Z(3)                      RAY11990
     *            + C33*Z(4) + C23*Z(5)                                 RAY12000
            Y(10) = B51*Z(6) + B52*Z(7) + B42*Z(8)                      RAY12010
     *            + B32*Z(9) + B22*Z(10)                                RAY12020
     *            + C51*Z(1) + C52*Z(2) + C42*Z(3)                      RAY12030
     *            + C32*Z(4) + C22*Z(5)                                 RAY12040
C                                                                       RAY12050
            IF( IGG.GT.2 )  THEN                                        RAY12060
              E1W = HK*(RAA*E5 + RBB*E6)                                RAY12070
              E3W = HK*(E5 + E6)                                        RAY12080
              E5W = HK*(E1 + B43)                                       RAY12090
              E6W = HK*(E1 + B34)                                       RAY12100
              F1W = E1W - E3W                                           RAY12110
              F2W = R2*F1W                                              RAY12120
              F3W = G1*F1W + E3W                                        RAY12130
              W33 = E1W                                                 RAY12140
              W34 = RAA*E3W                                             RAY12150
              W43 = RBB*E3W                                             RAY12160
              W25 =-R2*(F2W + R2*(E1W - RAA*W43))                       RAY12170
              W15 = RG*W25 + F2W                                        RAY12180
              W16 =-RG*W15 - F3W                                        RAY12190
              W22 = W16 + E1W                                           RAY12200
              W12 = RG*W16 - R4*F3W                                     RAY12210
              W52 =-RG*W12 + R4*(RG*F3W + R4*E3W)                       RAY12220
              W23 = R2*(E5W - RBB*E6W)                                  RAY12230
              W13 = RG*W23 - E5W                                        RAY12240
              W42 =-RG*W13 + R4*E5W                                     RAY12250
              W24 = R2*(E6W - RAA*E5W)                                  RAY12260
              W14 = RG*W24 - E6W                                        RAY12270
              W32 =-RG*W14 + R4*E6W                                     RAY12280
              W11 =-W16-W16                                             RAY12290
              W21 = W15+W15                                             RAY12300
              W31 = W14+W14                                             RAY12310
              W41 = W13+W13                                             RAY12320
              W51 = W12+W12                                             RAY12330
C                                                                       RAY12340
              Y(11) = B11*Z(11) + B12*Z(12) + B13*Z(13)                 RAY12350
     *              + B14*Z(14) + B15*Z(15)                             RAY12360
     1              + W11*Z( 1) + W12*Z( 2) + W13*Z( 3)                 RAY12370
     2              + W14*Z( 4) + W15*Z( 5)                             RAY12380
              Y(12) = B21*Z(11) + B22*Z(12) + B23*Z(13)                 RAY12390
     *              + B24*Z(14) + B25*Z(15)                             RAY12400
     1              + W21*Z( 1) + W22*Z( 2) + W23*Z( 3)                 RAY12410
     2              + W24*Z( 4) + W25*Z( 5)                             RAY12420
              Y(13) = B31*Z(11) + B32*Z(12) + B33*Z(13)                 RAY12430
     *              + B34*Z(14) + B24*Z(15)                             RAY12440
     1              + W31*Z( 1) + W32*Z( 2) + W33*Z( 3)                 RAY12450
     2              + W34*Z( 4) + W24*Z( 5)                             RAY12460
              Y(14) = B41*Z(11) + B42*Z(12) + B43*Z(13)                 RAY12470
     *              + B33*Z(14) + B23*Z(15)                             RAY12480
     1              + W41*Z( 1) + W42*Z( 2) + W43*Z( 3)                 RAY12490
     2              + W33*Z( 4) + W23*Z( 5)                             RAY12500
              Y(15) = B51*Z(11) + B52*Z(12) + B42*Z(13)                 RAY12510
     *              + B32*Z(14) + B22*Z(15)                             RAY12520
     1              + W51*Z( 1) + W52*Z( 2) + W42*Z( 3)                 RAY12530
     2              + W32*Z( 4) + W22*Z( 5)                             RAY12540
            ENDIF                                                       RAY12550
          ENDIF                                                         RAY12560
        ENDIF                                                           RAY12570
C                                                                       RAY12580
C     NORMALIZATION                                                     RAY12590
C                                                                       RAY12600
        Z1 = 0                                                          RAY12610
        DO  5  J=1,JX                                                   RAY12620
    5     Z1 = MAX(Z1,ABS(Y(J)))                                        RAY12630
        IF( Z1.GT.BIG )  THEN                                           RAY12640
          DO  6  J=1,JX                                                 RAY12650
    6       Y(J) = EPS*Y(J)                                             RAY12660
        ENDIF                                                           RAY12670
C                                                                       RAY12680
    7 CONTINUE                                                          RAY12690
C                                                                       RAY12700
C     NORMAL EXIT                                                       RAY12710
C                                                                       RAY12720
C     LIQUID SURFACE                                                    RAY12730
C                                                                       RAY12740
    8 IF( SV.LE.0 )  THEN                                               RAY12750
        Y0(1) = Y(1)                                                    RAY12760
        IF( ABS(Y(2))*EPS.LE.ABS(Y(1)) )  THEN                          RAY12770
          Y0(2) = Y(2)/ABS(Y(1))                                        RAY12780
        ELSE                                                            RAY12790
          Y0(2) = SIGN(BIG,Y(2))                                        RAY12800
        ENDIF                                                           RAY12810
C                                                                       RAY12820
        IF( IGG.GE.2 )  THEN                                            RAY12830
          EK =-WN*Y(4)/Y(1)                                             RAY12840
          IF( IGG.GT.2 )  U = C*Y(4)/(Y(4) + Y(6))                      RAY12850
        ENDIF                                                           RAY12860
C                                                                       RAY12870
C     SOLID SURFACE                                                     RAY12880
C                                                                       RAY12890
      ELSE                                                              RAY12900
        Y0(1) = Y(3)                                                    RAY12910
        IF( ABS(Y(5))*EPS.LE.ABS(Y(3)) )  THEN                          RAY12920
          Y0(2) = Y(5)/ABS(Y(3))                                        RAY12930
          Y0(3) =-Y(1)/Y(3)                                             RAY12940
        ELSE                                                            RAY12950
          Y0(2) = SIGN(BIG,Y(5))                                        RAY12960
        ENDIF                                                           RAY12970
C                                                                       RAY12980
        IF( IGG.GE.2 )  THEN                                            RAY12990
          EK =-WN*Y(10)/Y(3)                                            RAY13000
          IF( IGG.GT.2 )  U = C*Y(10)/(Y(10) + Y(15))                   RAY13010
        ENDIF                                                           RAY13020
      ENDIF                                                             RAY13030
      RETURN                                                            RAY13040
C                                                                       RAY13050
C     INPUT ERROR                                                       RAY13060
C                                                                       RAY13070
   90 WRITE(6,91)  L,W,C                                                RAY13080
   91   FORMAT(20X,5('?'),3X,'(RAYMRX)   INPUT ERROR',3X,               RAY13090
     *         'L  =',I5,3X,'W  =',1PE13.6,3X,                          RAY13100
     1         'C  =',E13.6,3X,5('?'))                                  RAY13110
      IER =-1                                                           RAY13120
C                                                                       RAY13130
C     DUMMY STATEMENTS                                                  RAY13140
C                                                                       RAY13150
      AP(1) = AP(1)                                                     RAY13160
      AE(1) = AE(1)                                                     RAY13170
      IA = IA                                                           RAY13180
      RETURN                                                            RAY13190
C                                                                       RAY13200
   92 WRITE(6,93)  I                                                    RAY13210
   93   FORMAT(20X,5('?'),3X,'(RAYMRX)   INPUT ERROR ',                 RAY13220
     *         'AT LAYER',I5,3X,5('?'))                                 RAY13230
      IER =-MAX(1,I)                                                    RAY13240
      RETURN                                                            RAY13250
      END                                                               RAY13260
