      program resamplec
c program to resample and select the microtremor data observed 
c independently & simultaneously.
c
c				                Jun,18,2008 T. Yokoi, IISEE, BRI
c
c Modified for double screening: by peak/rms and 
c                                by deviation of rms from average
c                                 May,12,2009 T. Yokoi, IISEE, BRI
c Modified for multiplexed input data file
c                                 Apr,13,2010 T. Yokoi, IISEE, BRI
c Modified for CCA array data     Apr,12,2017 T. Yokoi, IISEE, BRI
      parameter(nmax=500000,nch0=12,nblk0=8192,
     *          mblk0=2048,n_mea0=1000)
      character filen*80,cfn(n_mea0)*12,cout*12,comment*50,
     *          cform1*15,cform2*20,cform3*18,cform4*18
      character cdrv*10,cwkf*50,cunit*5,cname(n_mea0)*12
      real*4 x(nmax,nch0),xx(nmax),y(nblk0,nch0),peak(nch0),rmsa(nch0),
     *       rms(mblk0,nch0),aver(nch0),sgm(nch0)
      integer iflg(mblk0),mblkmea(n_mea0) 
      data cdrv/'./cca_wkf/'/ 
c default values
      data fl,fh,fs/0.1,10.,12./ ap,as/0.1,10.0/ ajudge/4./
     *     a_sgm/2/ ph0/0.0/
c Chebishev I(2), Lowpass (2) Bandpass(3), Causal(1)
      data ntype,nchara,ncausal/2,3,1/
c working folder name
      cwkf=cdrv//'prm/'
c temporary file 
      open(12,status='scratch')
c input data file list 
      filen=cdrv//'data/multiplexed_files/mltfile.lst'
      open(11,file=filen,status='unknown')
c
      filen=cwkf(1:14)//'resamplec.prm'
      write(6,'(a27)')filen
      open(10,file=filen,status='old')
c channel number, sampling interval, resampling rate, 
c             initial azimuth, center channel, radius
      read(10,*)nch,dt,nskip,ph0,nch_center,rr
      fnyquist=1./(2.*dt*real(nskip))
      fs=fnyquist*0.9
      fh=fs*0.9
c     fs=1.5
c     fh=1.0
      if(nskip.ge.2) then
       if(nchara.eq.2) then
      write(6,'( a9,2(a3,f8.3))')'Low-Pass:','fh=',fh,'fs=',fs
       else if(nchara.eq.3) then
      write(6,'(a10,2(a3,f8.3))')'Band-Pass:',
     *                              'fl=',fl,'fh=',fh,'fs=',fs
       endif
      endif
      dt1=dt*nskip
c     write(6,'(a4,f8.3)')'dt1=',dt1
      write(6,'(a4,i4)')'Nch=',nch
      write(6,'(a5,i4,a12,f8.3,a4,f8.3)')
     *         'Nskip=',nskip,' f(Nyquist)=',fnyquist,' fs=',fs
      call formatter(nch,cform1,cform2,cform3,cform4)
      read(10,*)ajudge,a_sgm
      write(6,'(2(a7,f6.1))')'ajudge=',ajudge,' a_sgm=',a_sgm
c start and end time 
      read(10,*) tst,tdur
      write(6,'(2(a6,f8.1))') ' tst =',tst,' tdur=',tdur
      nst0=int(tst/dt)+1
      ndur0=int(tdur/dt)
      scale=1.0
c output file name
      read(10,*)cout
c number of data in one time block after resampling
      read(10,*) nblk
c number of measurement in the same array configuration
      do i_mea=1,n_mea0 
       read(11,'(a12)',end=6)cname(i_mea)
       write(6,*)cname(i_mea)
      enddo
    6 n_mea=i_mea-1
      write(6,'(i3,a12)')n_mea,' measurement'
c
      write(6,'(a27,f5.1,a2)')
     *               ' First screening (peak/rms<',ajudge,'):'
      do i_mea=1,n_mea
       write(6,*)'i_mea=',i_mea
       ndat=ndur0/nskip
c input file names
       cfn(i_mea)=cname(i_mea)
       write(6,cform1)cfn(i_mea)
       cwkf=cdrv//'data/multiplexed_files/'
       filen=cwkf(1:33)//cfn(i_mea)
       write(6,*)i_mea,'-th measurement:',filen(1:45)
       open(1,file=filen,status='old')
       read(1,*)nch00,dt00,scale00,ndata00,cunit
c       write(6,*)nch00,dt00,scale00,ndata00,cunit
       if(nch00.ne.nch) then
	  write(6,*)' nch does not coincide, check the file.'
        stop
       endif
       if(dt00.ne.dt) then
	  write(6,*)' dt does not coincide, check the file.'
        stop
	 endif
       if(ndata00.lt.ndur0+nst0-1.or.nmax.lt.ndur0+nst0-1) then
	  write(6,*)' ndata00     =',ndata00
	  write(6,*)' nmax        =',nmax
	  write(6,*)' ndur0+nst0-1=',ndur0+nst0-1
        stop
	 endif
	 read(1,'(a50)')comment
c	 write(6,*)comment
       do i=1,nst0-1
        read(1,*)
       enddo
       do i=1,ndur0
        read(1,*,end=10) xdum,(x(i,j),j=1,nch)
c       if(abs(x(i,1)).lt.1.e-6) write(6,*) xdum,(x(i,j),j=1,3)
       enddo
   10  ndur0=i-1
c	 
       if(nskip.ge.2) then
        do j=1,nch
         do i=1,ndur0
          xx(i)=x(i,j)
         enddo
c        call taper(xx,ndur0,dt)
c
         nnave=nskip*nblk/20
         sum=0.0
         do i=1,nnave
          sum=sum+xx(i)
         enddo
         ave=sum/real(nnave)
         do i=1,ndur0
          xx(i)=xx(i)-ave
         enddo
         ntaper=nnave  
         do i=1,ntaper+1
          xx(i)=xx(i)*real(i-1)/real(ntaper)
         enddo         
c
         call bandp1(xx,ndur0,dt,fl,fh,fs,ap,as,ntype,nchara,ncausal)
c
         do i=1,ndat
          x(i,j)=xx((i-1)*nskip+1)
         enddo
        enddo
       endif
c first screening using the maximum value/rms amplitude
       iblkmx=0
       mblk=0
       iblk=0
       ist=0
  100  continue
c time block start and end
       if(ist.ne.0) then
        ist=ist+nblk/2
       else
        ist=1
       endif
       ied=ist+nblk-1
c escape from loop
       if(ied.gt.ndat) goto 200
       iblk=iblk+1
       do k=1,nblk
        do ich=1,nch
         y(k,ich)=x(ist+k-1,ich)
        enddo
       enddo
c peak value
       do ich=1,nch
        peak(ich)=abs(y(1,ich))
        do k=2,nblk
         if(peak(ich).lt.abs(y(k,ich))) peak(ich)=abs(y(k,ich))
        enddo
c rms amplitude
        sum=0.
        do k=1,nblk
         sum=sum+y(k,ich)**2
        enddo
        rmsa(ich)=sqrt(sum/nblk)
       enddo
c max. value of the ratio=peak/rms over channels
       aratio=peak(1)/rmsa(1)
       do ich=2,nch
        if(aratio.lt.peak(ich)/rmsa(ich)) aratio=peak(ich)/rmsa(ich)
       enddo
c screening
       iblkmx=iblkmx+1
       if(aratio.lt.ajudge) then
        iflg(iblk)=1
        mblk=mblk+1
       else
        iflg(iblk)=0
       endif
c
       goto 100
  200  continue
       write(6,*) mblk,' blocks remained among ',iblkmx,' blocks'
c store in the temporary file
       ist=0
       do iblk=1,iblkmx
        if(iblk.eq.1) then
         write(12,cform2)nch,mblk,ph0 ! ,(cfn(ich),ich=1,nch)
c        write(6,*)nch,mblk,ph0
        endif
c time block start and end
        if(ist.ne.0) then
         ist=ist+nblk/2
        else
         ist=1
        endif
        ied=ist+nblk-1
c
        if(iflg(iblk).eq.1) then
c     write(6,'(a4,f8.3)')'dt1=',dt1
	   do i=ist,ied
          t=dt1*real(i-1)
          write(12,cform3)i,t,(x(i,ich),ich=1,nch)
c          write(12)i,t,(x(i,ich),ich=1,nch)
         enddo
        endif
c end of do loop over iblk
       enddo
c end of do loop over i_mea
      enddo
      write(6,*)' Data stored in the temporary file'
c average and variance of rms amplitude
      rewind(12)
      mmblk=0
      do i_mea=1,n_mea
       read(12,'(4x,i4)') mblk
c       read(12) n_dum,mblk,adum,(cdum(ich),ich=1,nch)
       do iblk=1,mblk
        do k=1,nblk
         read(12,cform3)i,t,(y(k,ich),ich=1,nch)
c         write(6,*)iblk,k,i,t
c         read(12)i,t,(y(k,ich),ich=1,nch)
c         if(k.eq.1)    tst=t
c         if(k.eq.nblk) ted=t
        enddo
c rms amplitude
        do ich=1,nch
         sum=0.
         do k=1,nblk
          sum=sum+y(k,ich)**2 
         enddo
         rms(iblk+mmblk,ich)=sqrt(sum/real(nblk))
        enddo
c end of do loop over iblk
       enddo
       mmblk=mmblk+mblk
c end of do loop over i_mea
      enddo
c
      do ich=1,nch
       sum=0.
       do iblk=1,mmblk
        sum=sum+rms(iblk,ich)
       enddo
c average rms amplitude
       aver(ich)=sum/real(mmblk)
       sum=0.
       do iblk=1,mmblk
        sum=sum+(rms(iblk,ich)-aver(ich))**2
       enddo
       var=sum/real(mmblk-1)
c standard deviation
       sgm(ich)=sqrt(var)
      enddo
      mmblk0=mmblk
c
      cwkf=cdrv//'data/resampled_files/'
      filen=cwkf(1:31)//cout
      write(6,*)filen
      open(2,file=filen,status='unknown')
c screening using SD of rms amplitude
      rewind(12)
      icount=0
      iii=0
      do i_mea=1,n_mea
       read(12,'(4x,i4)') mblkmea(i_mea)
c       write(6,*) mblkmea(i_mea)
       do iblk=1,mblkmea(i_mea)
        do k=1,nblk
         read(12,cform4)t,(y(k,ich),ich=1,nch)
c         read(12)t,(y(k,ich),ich=1,nch)
         if(k.eq.1) tst=t
        enddo
c rms amplitude
        icount=icount+1
        iflg(icount)=1
        do ich=1,nch
         sum=0.
         do k=1,nblk
          sum=sum+y(k,ich)**2 
         enddo
         rrmss=sqrt(sum/real(nblk))
         if(abs(aver(ich)-rrmss).gt.a_sgm*sgm(ich)) then
          mmblk=mmblk-iflg(icount)
          iflg(icount)=0
         endif
        enddo
        if(iflg(icount).eq.1) iii=iii+1
c end of do loop over iblk
       enddo
c end of do loop over i_mea
      enddo
      if(icount.gt.mblk0) then
       write(6,*)'icount=',icount,', mblk0=',mblk0
       write(6,*)'icount>mblk0!, Use bigger mblk0.'
       stop
      endif
c	write(6,*)mmblk,iii,icount
      rewind(12)
      icount=0
      iii=0
      ncount=0
      do i_mea=1,n_mea
       read(12,'(4x,i4)') mblkmea(i_mea)
c       write(6,*) mblkmea(i_mea)
c       read(12) n_dum,m_dum,adum,(cdum(ich),ich=1,nch)
       do iblk=1,mblkmea(i_mea)
        ncount=ncount+1
        do k=1,nblk
         read(12,cform4)t,(y(k,ich),ich=1,nch)
c         read(12)t,(y(k,ich),ich=1,nch)
         if(k.eq.1) tst=t
        enddo
c       if(i_mea.eq.1.and.iblk.eq.1) then         !aca i_mea=1 & mblkmea(i_mea)=0
        if(ncount.eq.1.and.iblk.eq.1) then 
         write(2,'(i2,2i5,i2,2f8.2,1x,a18,1x,a5)') 
     *           nch,mmblk,nblk,nch_center,ph0,rr,cform3,cunit
        endif        
        icount=icount+1
        if(iflg(icount).eq.1) then
         iii=iii+1
         do k=1,nblk
          do ich=1,nch
          if(abs(y(k,ich)).gt.1.e5) then
           write(6,*)' Overflow! Use bigger scale factor.'
           stop
     	  endif
          enddo
         enddo
c     write(6,'(a4,f8.3)')'dt1=',dt1
         do k=1,nblk
          t=tst+real(k-1)*dt1
          write(2,cform3)k,t,(y(k,ich),ich=1,nch)
         enddo
        else
c        write(6,*)
c    *   ' A block is skipped at i_mea=',i_mea,' and iblk=',iblk
        endif
c end of do loop over iblk
       enddo
c end of do loop over i_mea
      enddo
      write(6,*)
     *' Second screening ({rms-average(rms)}/sigma<',a_sgm,'):'
      write(6,*)mmblk,' blocks remained among ',mmblk0,' blocks'
      close(12)
      close(2)
      close(10)
c
      stop
      end
c+++++
      include 'dgflt.f'
c+++++
      subroutine formatter(nch,cform1,cform2,cform3,cform4)
      character cform1*15,cform2*20,cform3*18,cform4*18
      if(nch.ge.10) then
       write(cform1,'(a4,i2,a9)')'(1x,',nch,'(a12,1x))'
       write(cform2,'(a10,10x)')'(2i4,f8.2)'
       write(cform3,'(a10,i2,a6)')'(i8,f16.4,',nch,'e15.7)'
       write(cform4,'(a10,i2,a6)')'(8x,f16.4,',nch,'e15.7)'
      else if(nch.ge.3) then
       write(cform1,'(a4,1x,i1,a9)')'(1x,',nch,'(a12,1x))'
       write(cform2,'(a10,10x)')'(2i4,f8.2)'
       write(cform3,'(a10,1x,i1,a6)')'(i8,f16.4,',nch,'e15.7)'
       write(cform4,'(a10,1x,i1,a6)')'(8x,f16.4,',nch,'e15.7)'
      else
       write(6,*)'nch should be bigger than 2!'
      endif
      end
c+++++
      subroutine taper(x,n,dt)
      real x(n)
      data pai/3.141592/
      mwin=int(real(n)/20.+0.5)
c linear trend fitting
      b1=x(1)
      do i=2,n
       b1=b1+real(i)*x(i)
      enddo
      b2=x(1)
      do i=1,n
       b2=b2+x(i)
      enddo
      a0=real(n*(n+1)*(2*n+1))/6.*dt
      b0=real(n*(n+1))/2.
      c0=b0*dt
      d0=real(n)
      f0=a0*d0-b0*c0
      aa=( d0*b1-b0*b2)/f0
      bb=(-c0*b1+a0*b2)/f0
      do i=1,n
       x(i)=x(i)-aa*real(i)*dt-bb
      enddo
      return
      end
c+++++
